package ru.dbotthepony.mc.otm.menu;

import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.ItemStack;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.block.entity.BlockEntityDriveViewer;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.drive.IMatteryDrive;
import ru.dbotthepony.mc.otm.menu.data.NetworkedItemView;
import ru.dbotthepony.mc.otm.menu.data.UUIDDataContainer;
import ru.dbotthepony.mc.otm.menu.slot.MatterySlot;

import javax.annotation.Nullable;

public class DriveViewerMenu extends PoweredMatteryMenu {
	public UUIDDataContainer view_uuid;
	public NetworkedItemView view;
	public MatterySlot drive_slot;
	protected IMatteryDrive last_drive;

	public NetworkedItemView getView() {
		return view != null ? view : NetworkedItemView.getRemote(view_uuid.getValue());
	}

	public DriveViewerMenu(int containerID, Inventory inventory) {
		this(containerID, inventory, null);
	}

	public DriveViewerMenu(int containerID, Inventory inventory, @Nullable BlockEntityDriveViewer tile) {
		super(Registry.Menus.DRIVE_VIEWER, containerID, inventory, tile);

		view_uuid = new UUIDDataContainer();
		var container = tile != null ? tile.drive_slot : new SimpleContainer(1);

		drive_slot = new MatterySlot(container, 0) {
			@Override
			public boolean mayPlace(ItemStack stack) {
				return stack.getCapability(MatteryCapability.DRIVE).isPresent();
			}
		};

		addSlot(drive_slot);

		if (tile != null) {
			view = new NetworkedItemView(inventory.player);
			view_uuid.setValue(view.id);
		}

		addDataSlots(view_uuid);

		addBatterySlot();
		addInventorySlots();
	}

	@Override
	public void broadcastChanges() {
		super.broadcastChanges();

		if (tile != null) {
			var get = ((BlockEntityDriveViewer) tile).drive_slot.getItem(0);

			if (get.isEmpty()) {
				if (last_drive != null) {
					view.clear();
				}

				last_drive = null;
			} else {
				var get_cap = get.getCapability(MatteryCapability.DRIVE).resolve();

				if (get_cap.isEmpty()) {
					if (last_drive != null) {
						view.clear();
					}

					last_drive = null;
				} else {
					if (last_drive != get_cap.get()) {
						view.clear();

						for (var stack : get_cap.get().getItems()) {
							view.addItem(stack.stack(), stack.id());
						}
					}

					last_drive = get_cap.get();
				}
			}

			view.network();
		}
	}

	@Override
	public void removed(Player p_38940_) {
		super.removed(p_38940_);

		if (!p_38940_.level.isClientSide) {
			var view = getView();

			if (view != null)
				view.remove();
		}
	}

	@Override
	protected int getWorkingSlotStart() {
		return 0;
	}

	@Override
	protected int getWorkingSlotEnd() {
		return 2;
	}
}
